@@ -11,6 +11,7 @@ namespace PHPUnit\TextUI;
 
 use const PHP_EOL;
 use const PHP_VERSION;
+use function assert;
 use function class_exists;
 use function explode;
 use function function_exists;
@@ -53,6 +54,7 @@ use PHPUnit\Runner\ResultCache\ResultCache;
 use PHPUnit\Runner\ResultCache\ResultCacheHandler;
 use PHPUnit\Runner\TestSuiteSorter;
 use PHPUnit\Runner\Version;
+use PHPUnit\TestRunner\IssueFilter;
 use PHPUnit\TestRunner\TestResult\Facade as TestResultFacade;
 use PHPUnit\TextUI\CliArguments\Builder;
 use PHPUnit\TextUI\CliArguments\Configuration as CliConfiguration;
@@ -82,6 +84,7 @@ use PHPUnit\TextUI\Output\Printer;
 use PHPUnit\TextUI\XmlConfiguration\Configuration as XmlConfiguration;
 use PHPUnit\TextUI\XmlConfiguration\DefaultConfiguration;
 use PHPUnit\TextUI\XmlConfiguration\Loader;
+use PHPUnit\Util\Http\PhpDownloader;
 use SebastianBergmann\Timer\Timer;
 use Throwable;
 
@@ -120,14 +123,6 @@ final readonly class Application
 
             $this->executeCommandsThatDoNotRequireTheTestSuite($configuration, $cliConfiguration);
 
-            $testSuite = $this->buildTestSuite($configuration);
-
-            $this->executeCommandsThatRequireTheTestSuite($configuration, $cliConfiguration, $testSuite);
-
-            if ($testSuite->isEmpty() && !$configuration->hasCliArguments() && $configuration->testSuite()->isEmpty()) {
-                $this->execute(new ShowHelpCommand(Result::FAILURE));
-            }
-
             $pharExtensions                          = null;
             $extensionRequiresCodeCoverageCollection = false;
             $extensionReplacesOutput                 = false;
@@ -148,26 +143,12 @@ final readonly class Application
                 $extensionReplacesResultOutput           = $bootstrappedExtensions['replacesResultOutput'];
             }
 
-            CodeCoverage::instance()->init(
-                $configuration,
-                CodeCoverageFilterRegistry::instance(),
-                $extensionRequiresCodeCoverageCollection,
-            );
-
             $printer = OutputFacade::init(
                 $configuration,
                 $extensionReplacesProgressOutput,
                 $extensionReplacesResultOutput,
             );
 
-            if (!$configuration->debug() && !$extensionReplacesOutput) {
-                $this->writeRuntimeInformation($printer, $configuration);
-                $this->writePharExtensionInformation($printer, $pharExtensions);
-                $this->writeRandomSeedInformation($printer, $configuration);
-
-                $printer->print(PHP_EOL);
-            }
-
             if ($configuration->debug()) {
                 EventFacade::instance()->registerTracer(
                     new EventLogger(
@@ -177,13 +158,13 @@ final readonly class Application
                 );
             }
 
+            TestResultFacade::init();
+            DeprecationCollector::init();
+
             $this->registerLogfileWriters($configuration);
 
             $testDoxResultCollector = $this->testDoxResultCollector($configuration);
 
-            TestResultFacade::init();
-            DeprecationCollector::init();
-
             $resultCache = $this->initializeTestResultCache($configuration);
 
             if ($configuration->controlGarbageCollector()) {
@@ -195,10 +176,32 @@ final readonly class Application
 
             $baselineGenerator = $this->configureBaseline($configuration);
 
-            $this->configureDeprecationTriggers($configuration);
-
             EventFacade::instance()->seal();
 
+            $testSuite = $this->buildTestSuite($configuration);
+
+            $this->executeCommandsThatRequireTheTestSuite($configuration, $cliConfiguration, $testSuite);
+
+            if ($testSuite->isEmpty() && !$configuration->hasCliArguments() && $configuration->testSuite()->isEmpty()) {
+                $this->execute(new ShowHelpCommand(Result::FAILURE));
+            }
+
+            CodeCoverage::instance()->init(
+                $configuration,
+                CodeCoverageFilterRegistry::instance(),
+                $extensionRequiresCodeCoverageCollection,
+            );
+
+            if (!$configuration->debug() && !$extensionReplacesOutput) {
+                $this->writeRuntimeInformation($printer, $configuration);
+                $this->writePharExtensionInformation($printer, $pharExtensions);
+                $this->writeRandomSeedInformation($printer, $configuration);
+
+                $printer->print(PHP_EOL);
+            }
+
+            $this->configureDeprecationTriggers($configuration);
+
             $timer = new Timer;
             $timer->start();
 
@@ -255,7 +258,12 @@ final readonly class Application
             $result = TestResultFacade::result();
 
             if (!$extensionReplacesResultOutput && !$configuration->debug()) {
-                OutputFacade::printResult($result, $testDoxResult, $duration);
+                OutputFacade::printResult(
+                    $result,
+                    $testDoxResult,
+                    $duration,
+                    $configuration->hasSpecificDeprecationToStopOn(),
+                );
             }
 
             CodeCoverage::instance()->generateReports($printer, $configuration);
@@ -276,6 +284,7 @@ final readonly class Application
 
             $shellExitCode = (new ShellExitCodeCalculator)->calculate(
                 $configuration->failOnDeprecation(),
+                $configuration->failOnPhpunitDeprecation(),
                 $configuration->failOnEmptyTestSuite(),
                 $configuration->failOnIncomplete(),
                 $configuration->failOnNotice(),
@@ -297,38 +306,41 @@ final readonly class Application
 
     private function execute(Command\Command $command, bool $requiresResultCollectedFromEvents = false): never
     {
+        $errored = false;
+
         if ($requiresResultCollectedFromEvents) {
             try {
                 TestResultFacade::init();
                 EventFacade::instance()->seal();
 
                 $resultCollectedFromEvents = TestResultFacade::result();
+
+                $errored = $resultCollectedFromEvents->hasTestTriggeredPhpunitErrorEvents();
             } catch (EventFacadeIsSealedException|UnknownSubscriberTypeException) {
             }
         }
 
         print Version::getVersionString() . PHP_EOL . PHP_EOL;
 
-        $result = $command->execute();
+        if (!$errored) {
+            $result = $command->execute();
 
-        print $result->output();
+            print $result->output();
 
-        $shellExitCode = $result->shellExitCode();
+            exit($result->shellExitCode());
+        }
 
-        if (isset($resultCollectedFromEvents) &&
-            $resultCollectedFromEvents->hasTestTriggeredPhpunitErrorEvents()) {
-            $shellExitCode = Result::EXCEPTION;
+        assert(isset($resultCollectedFromEvents));
 
-            print PHP_EOL . PHP_EOL . 'There were errors:' . PHP_EOL;
+        print 'There were errors:' . PHP_EOL;
 
-            foreach ($resultCollectedFromEvents->testTriggeredPhpunitErrorEvents() as $events) {
-                foreach ($events as $event) {
-                    print PHP_EOL . trim($event->message()) . PHP_EOL;
-                }
+        foreach ($resultCollectedFromEvents->testTriggeredPhpunitErrorEvents() as $events) {
+            foreach ($events as $event) {
+                print PHP_EOL . trim($event->message()) . PHP_EOL;
             }
         }
 
-        exit($shellExitCode);
+        exit(Result::EXCEPTION);
     }
 
     private function loadBootstrapScript(string $filename): void
@@ -459,7 +471,7 @@ final readonly class Application
         }
 
         if ($cliConfiguration->checkVersion()) {
-            $this->execute(new VersionCheckCommand);
+            $this->execute(new VersionCheckCommand(new PhpDownloader, Version::majorVersionNumber(), Version::id()));
         }
 
         if ($cliConfiguration->help()) {
@@ -469,10 +481,6 @@ final readonly class Application
 
     private function executeCommandsThatDoNotRequireTheTestSuite(Configuration $configuration, CliConfiguration $cliConfiguration): void
     {
-        if ($cliConfiguration->listSuites()) {
-            $this->execute(new ListTestSuitesCommand($configuration->testSuite()));
-        }
-
         if ($cliConfiguration->warmCoverageCache()) {
             $this->execute(new WarmCodeCoverageCacheCommand($configuration, CodeCoverageFilterRegistry::instance()));
         }
@@ -480,6 +488,10 @@ final readonly class Application
 
     private function executeCommandsThatRequireTheTestSuite(Configuration $configuration, CliConfiguration $cliConfiguration, TestSuite $testSuite): void
     {
+        if ($cliConfiguration->listSuites()) {
+            $this->execute(new ListTestSuitesCommand($testSuite));
+        }
+
         if ($cliConfiguration->listGroups()) {
             $this->execute(
                 new ListGroupsCommand(
@@ -669,7 +681,10 @@ final readonly class Application
         if ($configuration->hasLogfileTestdoxHtml() ||
             $configuration->hasLogfileTestdoxText() ||
             $configuration->outputIsTestDox()) {
-            return new TestDoxResultCollector(EventFacade::instance());
+            return new TestDoxResultCollector(
+                EventFacade::instance(),
+                new IssueFilter($configuration->source()),
+            );
         }
 
         return null;
