@@ -7,7 +7,7 @@
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
-namespace PHPUnit\SelfTest\Basic;
+namespace PHPUnit\TestFixture\Basic;
 
 use PHPUnit\Framework\TestCase;
 use RuntimeException;
